package cn.fw.hermes.common.utils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author 张志伟
 * @date 2018-02-24 15:12
 * @description
 */
public final class DateUtils {
    public static final long SECOND = 1000L;
    public static final long MINUTE = 60000L;
    public static final long HOUR = 3600000L;
    public static final long DAY = 86400000L;
    public static final long WEEK = 604800000L;
    public static final long YEAR = 31536000000L;
    public static final String FOMTER_TIMES = "yyyy-MM-dd HH:mm:ss";
    private static final Map<Integer, String> WEEK_DAY = new HashMap();
    private static final Map<String, Integer> DATE_FIELD;

    static {
        WEEK_DAY.put(Integer.valueOf(7), "星期六");
        WEEK_DAY.put(Integer.valueOf(1), "星期天");
        WEEK_DAY.put(Integer.valueOf(2), "星期一");
        WEEK_DAY.put(Integer.valueOf(3), "星期二");
        WEEK_DAY.put(Integer.valueOf(4), "星期三");
        WEEK_DAY.put(Integer.valueOf(5), "星期四");
        WEEK_DAY.put(Integer.valueOf(6), "星期五");
        DATE_FIELD = new HashMap();
        DATE_FIELD.put("y", Integer.valueOf(1));
        DATE_FIELD.put("M", Integer.valueOf(2));
        DATE_FIELD.put("d", Integer.valueOf(5));
        DATE_FIELD.put("H", Integer.valueOf(10));
        DATE_FIELD.put("m", Integer.valueOf(12));
        DATE_FIELD.put("s", Integer.valueOf(13));
        DATE_FIELD.put("w", Integer.valueOf(4));
    }

    /**
     * 指定格式的字符串转date
     * @param date
     * @param pattern
     * @return
     */
    public static Date parse(String date, String pattern) {
        Date resultDate = null;
        try {
            resultDate = new SimpleDateFormat(pattern).parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return resultDate;
    }

    /**
     * 数据库的时间格式Timestamp
     * @param date
     * @return
     */
    public static Timestamp parseSimple(String date) {
        Date result = null;
        try {
            DateFormat yyyyMMdd = new SimpleDateFormat("yyyy-MM-dd");
            result = yyyyMMdd.parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result != null ? new Timestamp(result.getTime()) : null;
    }

    public static Timestamp parseFull(String date) {
        Date result = null;
        try {
            DateFormat yyyyMMddHHmmss = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            result = yyyyMMddHHmmss.parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result != null ? new Timestamp(result.getTime()) : null;
    }

    public static Timestamp parse(Object object) {
        if ((object instanceof Date)) {
            return new Timestamp(((Date) object).getTime());
        }
        if (StringUtils.isEmpty(object)) {
            return null;
        }
        String date = StringUtils.asString(object);
        try {
            if (date.length() == 10) {
                return parseSimple(date);
            }
            if (date.length() == 8) {
                DateFormat yyyyMMdd = new SimpleDateFormat("yyyyMMdd");
                Date d = yyyyMMdd.parse(date);
                return new Timestamp(d.getTime());
            }
            if (date.length() == 9) {
                if (date.matches("\\d{4}-\\d-\\d{2}")) {
                    DateFormat yyyyMdd = new SimpleDateFormat("yyyy-M-dd");
                    Date d = yyyyMdd.parse(date);
                    return new Timestamp(d.getTime());
                }
                if (date.matches("\\d{4}-\\d{2}-\\d")) {
                    DateFormat yyyyMdd = new SimpleDateFormat("yyyy-MM-d");
                    Date d = yyyyMdd.parse(date);
                    return new Timestamp(d.getTime());
                }
            } else {
                if (date.length() == 16) {
                    DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                    Date d = format.parse(date);
                    return new Timestamp(d.getTime());
                }
                if (date.length() == 18) {
                    DateFormat format = new SimpleDateFormat("yyyy-MM-ddHH:mm:ss");
                    Date d = format.parse(date);
                    return new Timestamp(d.getTime());
                }
                if (date.length() == 19) {
                    return parseFull(date);
                }
                if ((date.length() >= 20) && (date.length() <= 23)) {
                    if (date.matches("\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}\\.\\d{1,3}")) {
                        DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
                        Date d = format.parse(date);
                        return new Timestamp(d.getTime());
                    }
                } else {
                    return Timestamp.valueOf(object.toString());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 将时间转换成制定格式的字符串
     * @param date
     * @param pattern
     * @return
     */
    public static String format(Date date, String pattern) {
        if (date == null) {
            return null;
        }
        DateFormat format = new SimpleDateFormat(pattern);
        return format.format(date);
    }

    /**
     * 默认将时间转成xxxx年xx月xx日
     * @param date
     * @return
     */
    public static String formatCHS(Date date) {
        if (date == null) {
            return null;
        }
        DateFormat format = new SimpleDateFormat("yyyy年MM月dd日");
        return format.format(date);
    }

    /**
     * 默认将时间转成yyyy-MM-dd格式
     * @param date
     * @return
     */
    public static String format(Date date) {
        if (date == null) {
            return null;
        }
        DateFormat YYYY_MM_DD = new SimpleDateFormat("yyyy-MM-dd");
        return YYYY_MM_DD.format(date);
    }

    /**
     * 默认将时间转成yyyy-MM-dd HH:mm:ss格式
     * @param date
     * @return
     */
    public static String formatFull(Date date) {
        DateFormat YYYY_MM_DD_HH_MM_SS = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return YYYY_MM_DD_HH_MM_SS.format(date);
    }

    /**
     * 获得当前时间戳
     * @return
     */
    public static Timestamp getNow() {
        return new Timestamp(System.currentTimeMillis());
    }

    /**
     * 取得当前的年份
     * @return
     */
    public static Integer getNowYear() {
        return getYear(getNow());
    }

    /**
     * 取得当前的月份
     * @return
     */
    public static Integer getNowMonth() {
        return getMonth(getNow());
    }

    /**
     * 取得date的年份
     * @param value
     * @return
     */
    public static Integer getYear(Object value) {
        Calendar c = Calendar.getInstance();
        Date date = getDate(value);
        c.setTime(date);
        return Integer.valueOf(c.get(1));
    }

    /**
     * 取得月份
     * @param value
     * @return
     */
    public static Integer getMonth(Object value) {
        Calendar c = Calendar.getInstance();
        Date date = getDate(value);
        c.setTime(date);
        return Integer.valueOf(c.get(2) + 1);
    }

    /**
     * 取得天数
     * @param value
     * @return
     */
    public static Integer getDay(Object value) {
        Calendar c = Calendar.getInstance();
        Date date = getDate(value);
        c.setTime(date);
        return Integer.valueOf(c.get(5));
    }

    /**
     * 对象转换成Date类型
     * @param value
     * @return
     */
    private static Date getDate(Object value) {
        Date date = null;
        if ((value instanceof Date)) {
            date = (Date) value;
        } else {
            date = parse((String) value);
        }
        if (date == null) {
            throw new RuntimeException("日期格式解析错误!date=" + value);
        }
        return date;
    }

    /**
     * 输入出生年月计算年龄
     * @param value
     * @return
     */
    public static Integer getAge(Object value) {
        Timestamp date = parse(value);
        if (date == null) {
            return null;
        }
        Timestamp now = getNow();
        int year1 = getYear(date);
        int year2 = getNowYear();
        int month1 = getMonth(date);
        int month2 = getNowMonth();
        int date1 = getDay(date);
        int date2 = getDay(getNow());
        int months = (year2 - year1) * 12 + month2 - month1;
        if (date1 > date2) {
            months -= 1;
        }
        Integer age = months / 12;
        return age;
    }

    /**
     * 计算x年前/后的时间
     * @param offsetYear
     * @return
     */
    public static Timestamp getNowExpiredYear(int offsetYear) {
        Calendar now = Calendar.getInstance();
        now.add(1, offsetYear);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * 计算x月前/后的时间
     * @param offset
     * @return
     */
    public static Timestamp getNowExpiredMonth(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(2, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * 计算X天前/后的时间
     * @param offset
     * @return
     */
    public static Timestamp getNowExpiredDay(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(5, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * 计算x小时前/后的时间
     * @param offset
     * @return
     */
    public static Timestamp getNowExpiredHour(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(10, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * 计算x秒前/后的时间
     * @param offsetSecond
     * @return
     */
    public static Timestamp getNowExpiredSecond(int offsetSecond) {
        Calendar now = Calendar.getInstance();
        now.add(13, offsetSecond);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * 计算x分钟前/后的时间
     * @param offset
     * @return
     */
    public static Timestamp getNowExpiredMinute(int offset) {
        Calendar now = Calendar.getInstance();
        now.add(12, offset);
        return new Timestamp(now.getTime().getTime());
    }

    /**
     * 计算与A时间X天前/后的时间
     * @param givenDate
     * @param offset
     * @return
     */
    public static Timestamp getExpiredDay(Date givenDate, int offset) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(5, offset);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * 计算与A时间X月前/后的时间
     * @param givenDate
     * @param offset
     * @return
     */
    public static Timestamp getExpiredMonth(Date givenDate, int offset) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(2, offset);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * 计算与A时间X年前/后的时间
     * @param givenDate
     * @param offsetHour
     * @return
     */
    public static Timestamp getExpiredYear(Date givenDate, int offsetHour) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(1, offsetHour);
        return new Timestamp(date.getTime().getTime());
    }

    /**
     * 计算与A时间X秒前/后的时间
     * @param givenDate
     * @param second
     * @return
     */
    public static Timestamp getExpiredSecond(Date givenDate, int second) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(13, second);
        return new Timestamp(date.getTime().getTime());
    }

    public static Timestamp getExpired(Date givenDate, int offset, Integer type) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(type.intValue(), offset);
        return new Timestamp(date.getTime().getTime());
    }

    public static Calendar getCalendar(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c;
    }

    public static Timestamp getExpiredHour(Date givenDate, int offsetHour) {
        Calendar date = Calendar.getInstance();
        date.setTime(givenDate);
        date.add(10, offsetHour);
        return new Timestamp(date.getTime().getTime());
    }

    public static Date getMonthFirstDay(Date givenDate) {
        Date date = parse(format(givenDate, "yyyy-MM"), "yyyy-MM");
        return date;
    }

    public static int getDayOfWeek(Date givenDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(givenDate);
        int day = c.get(7);
        return day;
    }

    public static String getChineseDayOfWeek(int dayOfWeek) {
        return (String) WEEK_DAY.get(Integer.valueOf(dayOfWeek));
    }

    public static Boolean between(Date date, Date begin, Date end) {
        if ((date == null) || (begin == null) || (end == null)) {
            return Boolean.valueOf(true);
        }
        return Boolean.valueOf((date.after(begin)) && (date.before(end)));
    }

    public static Boolean between(Date begin, Date end) {
        Date now = getNow();
        return between(now, begin, end);
    }

    public static Calendar getTodayZero() {
        Calendar c = Calendar.getInstance();
        c.set(11, 0);
        c.set(12, 0);
        c.set(13, 0);
        c.set(14, 0);
        return c;
    }

    public static boolean isDate(String value) {
        return parse(value) != null;
    }

    public static Date parseDate(String str, String[] parsePatterns) throws ParseException {
        if ((str == null) || (parsePatterns == null)) {
            throw new IllegalArgumentException("Date and Patterns must not be null");
        }
        SimpleDateFormat parser = null;
        ParsePosition pos = new ParsePosition(0);
        for (int i = 0; i < parsePatterns.length; i++) {
            if (i == 0) {
                parser = new SimpleDateFormat(parsePatterns[0]);
            } else {
                parser.applyPattern(parsePatterns[i]);
            }
            pos.setIndex(0);
            Date date = parser.parse(str, pos);
            if ((date != null) && (pos.getIndex() == str.length())) {
                return date;
            }
        }
        throw new ParseException("Unable to parse the date: " + str, -1);
    }

    public static java.sql.Date getSqlDate(Object value) {
        if (value == null) {
            return null;
        }
        if ((value instanceof Date)) {
            return new java.sql.Date(((Date) value).getTime());
        }
        if ((value instanceof String)) {
            Timestamp date = parse((String) value);
            return date != null ? new java.sql.Date(date.getTime()) : null;
        }
        return null;
    }

    public static Map<String, Object> getDateVariableMap() {
        Map<String, Object> dateMap = new HashMap();
        Date now = getNow();
        dateMap.put("now", now);
        dateMap.put("year", getYear(now));
        dateMap.put("month", getMonth(now));
        dateMap.put("day", getDay(now));
        dateMap.put("simple", format(now));
        dateMap.put("full", formatFull(now));
        dateMap.put("chs", formatCHS(now));
        dateMap.put("year_ago", Integer.valueOf(getYear(now).intValue() - 1));
        dateMap.put("month_next", getExpiredMonth(now, 1));
        dateMap.put("month_pre", getExpiredMonth(now, -1));
        dateMap.put("quarter_next", getExpiredMonth(now, 3));
        dateMap.put("quarter_pre", getExpiredMonth(now, -3));
        dateMap.put("year_next", getExpiredMonth(now, 12));
        dateMap.put("year_pre", getExpiredMonth(now, -12));
        dateMap.put("day_next", getExpiredDay(now, 1));
        dateMap.put("day_pre", getExpiredDay(now, -1));
        dateMap.put("day_pre_simple", format(getExpiredDay(now, -1)));
        dateMap.put("day_net_week_simple", format(getExpiredDay(now, 7)));
        dateMap.put("day_pre_week_simple", format(getExpiredDay(now, -7)));
        int month = getMonth(now).intValue();
        int year = getYear(now).intValue();
        int yearjs = month > 9 ? year + 1 : year;
        dateMap.put("yearjs", Integer.valueOf(yearjs));
        dateMap.put("yearjs_pre", Integer.valueOf(yearjs - 1));
        dateMap.put("yearjs_next", Integer.valueOf(yearjs + 1));
        return dateMap;
    }

    /**
     * 一个日期上加上几天
     *
     * @param date 某一天
     * @return Date 加一天后的时间
     */
    public static Date addSomeDay(Date date, Integer day) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, day);
        return calendar.getTime();
    }

    /**
     * 取得一天的开始时间
     *
     * @param date 某一天时间
     * @return Date符合格式的这天开始时间
     */
    public static Date dayBegin(Date date) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 取得一天的结束时间
     *
     * @param date 某一天时间
     * @return Date符合格式的这天结束时间
     */
    public static Date dayEnd(Date date) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar.getTime();
    }

    public static Date getExpired(Object givenDate, String off) {
        Date date = null;
        if ((givenDate instanceof Date)) {
            date = (Date) givenDate;
        } else {
            date = parse(StringUtils.asString(givenDate));
        }
        String op = String.valueOf(off.charAt(0));
        int offset = Integer.parseInt(String.valueOf(off.charAt(1)));
        String type = String.valueOf(off.charAt(2));
        Date result = null;
        int dateField = ((Integer) DATE_FIELD.get(type)).intValue();
        if (op.equals("+")) {
            result = getExpired(date, offset, Integer.valueOf(dateField));
        } else if (op.equals("-")) {
            result = getExpired(date, -offset, Integer.valueOf(dateField));
        }
        return result;
    }

    public static Date eval(String expr) {
        expr = expr.trim();
        if ((expr.equals("today")) || (expr.equals("now"))) {
            return getNow();
        }
        String EXPR = "(.+)\\s*(\\+|\\-)\\s*(\\w+)";
        String p1 = expr.replaceAll(EXPR, "$1").trim();
        String p2 = expr.replaceAll(EXPR, "$2").trim();
        String p3 = expr.replaceAll(EXPR, "$3").trim();
        Date date = null;
        if (p1.equals("today")) {
            date = getNow();
        } else {
            p1 = p1.replace("'", "").trim();
            date = parse(p1);
        }
        Date result = null;
        result = getExpired(date, p2 + p3);
        return result;
    }

    public static Integer sub(Object a, Object b, String type) {
        Date d1 = parse(a);
        Date d2 = parse(b);
        if ((d1 == null) || (d2 == null)) {
            return null;
        }
        Long result = null;
        long DAY = 86400000L;
        long sub = d1.getTime() - d2.getTime();
        long daysub = sub / DAY;
        long y1 = getYear(d1);
        long y2 = getYear(d2);
        long m1 = getMonth(d1);
        long m2 = getMonth(d2);
        long monthsub = (y1 - y2) * 12L + (m1 - m2);
        if (type.equals("m")) {
            result = Long.valueOf(monthsub);
        } else if (type == "y") {
            result = Long.valueOf(monthsub / 12L);
        } else if (type.equals("d")) {
            result = Long.valueOf(daysub);
        }
        return Integer.valueOf(result.intValue());
    }

    public static void main(String[] args) {
        System.out.println(getYear(new Date()));
        System.out.println(getMonth(new Date()));
        System.out.println(eval("today + 1d"));
        System.out.println(eval("today - 1d"));
        System.out.println(eval("today + 1M"));
        System.out.println(eval("today - 1M"));
        System.out.println(eval("today + 1y"));
        System.out.println(eval("today - 1y"));
        System.out.println(eval("2014-06-29 + 1d"));
        System.out.println(eval("2014-06-29 - 1d"));
        System.out.println(eval("2014-06-29 + 1M"));
        System.out.println(eval("2014-06-29 - 1M"));
        System.out.println(eval("2014-06-29 + 1y"));
        System.out.println(eval("2014-06-29 - 1y"));
        System.out.println(eval("2014-06-29 - 1w"));
        System.out.println(getExpired(new Date(), "-1M"));
        String now = getNow().toString();
        System.out.println(now + "=" + now + " parse=" + parse(now) + "");
        System.out.println(sub("2014-07-30", "2013-06-29", "m"));
        System.out.println(getAge("1991-10-07"));
        System.out.println(parse("2015-03-2519:35:21"));
    }
}
